﻿using HoloToolkit.Unity;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
#if UNITY_UWP && !UNITY_EDITOR
using System;
using Windows.Storage;
using Windows.System;
using System.Threading.Tasks;
using Windows.Storage.Streams;
#endif
public class FileObjectsManager : Singleton<FileObjectsManager> {

#if UNITY_UWP && !UNITY_EDITOR
    Windows.Storage.ApplicationDataContainer localSettings = Windows.Storage.ApplicationData.Current.LocalSettings;
    Windows.Storage.StorageFolder localFolder = Windows.Storage.ApplicationData.Current.LocalFolder;
#endif

    [Tooltip("Objects file name in disk")]
    public string file_name = "objects.json";
    
    bool isLoadedObjectsMsgSent = false;

    bool isIntialized = false;

    [Tooltip("Clean Files and anchors on start")]
    public bool cleanFileOnStart = false;

    Dictionary<string, List<string>> loadedObjects;

    void Start()
    {
        MySocket.initConnection();
        loadedObjects = new Dictionary<string, List<string>>();
        initFileManager();
    }

    void Update()
    {
        if (isIntialized && !isLoadedObjectsMsgSent)
        {
            RealWorldObjectManager.Instance.onObjectsLoadedFromDisk(loadedObjects);
            isLoadedObjectsMsgSent = true;
        }
    }

#if UNITY_UWP && !UNITY_EDITOR
    async
#endif
    void initFileManager()
    {
#if UNITY_UWP && !UNITY_EDITOR
        var x = await localFolder.TryGetItemAsync(file_name);
        if (x != null)
        {
            if (cleanFileOnStart) {
                StorageFile sampleFile = await localFolder.GetFileAsync(file_name);
                await sampleFile.DeleteAsync();
            }
            else 
            {
                StorageFile sampleFile = await localFolder.GetFileAsync(file_name);
                string jsonData = await FileIO.ReadTextAsync(sampleFile);
                loadedObjects = Newtonsoft.Json.JsonConvert.DeserializeObject<Dictionary<string, List<string>>>(jsonData);
            }
        }
        isIntialized = true;
#endif
    }

#if UNITY_UWP && !UNITY_EDITOR
    async
#endif
    public void writeToFileManager(Dictionary<string, List<string>> dict)
    {
#if UNITY_UWP && !UNITY_EDITOR
        string msg = "";
        msg = Newtonsoft.Json.JsonConvert.SerializeObject(dict);
        var x = await localFolder.TryGetItemAsync(file_name);
        if (x == null)
        {
            await localFolder.CreateFileAsync(file_name,
            CreationCollisionOption.ReplaceExisting);
        }
        var sampleFile = await localFolder.GetFileAsync(file_name);
        await FileIO.WriteTextAsync(sampleFile, msg);
#endif
    }
}
