﻿using HoloToolkit.Unity;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.Windows.Speech;

public class FindWorldObjects : MonoBehaviour {

    TextToSpeech textToSpeech;
    KeywordRecognizer keywordRecognizer = null;
    State currentState;
    ObjectTracking tracker;

    string command = "Find";
    string backCmd = "Back";

    enum State
    {
        Wake, Play
    };

    // Use this for initialization
    void Start () {

        currentState = State.Wake;
        var worldObjLst = RealWorldObjectManager.Instance.getWorldObjects();
        var keywords = worldObjLst.Select(s => command + " " + s).ToList();
        keywords.Add(backCmd);
        keywordRecognizer = new KeywordRecognizer(keywords.ToArray());
        keywordRecognizer.OnPhraseRecognized += KeywordRecognizer_OnPhraseRecognized;
        keywordRecognizer.Start();

    }

    private void Awake()
    {
        tracker = GetComponent<ObjectTracking>();
        textToSpeech = GetComponent<TextToSpeech>();
    }

    // Update is called once per frame
    void Update () {
        if (currentState == State.Wake)
        {
            StartCoroutine(Speak("Say Find with the Object Name you wish to find."
            , () => { }));
            currentState = State.Play;
        }
        if (!keywordRecognizer.IsRunning)
        {
            keywordRecognizer.Start();
        }
    }

    string lastObjectName = null;

    private void KeywordRecognizer_OnPhraseRecognized(PhraseRecognizedEventArgs args)
    {
        if (args.text == backCmd)
        {
            goBack();
            return;
        }
        string objectName = args.text.Remove(0, (command + " ").Length);
        lastObjectName = objectName;
        List<GameObject> lst = RealWorldObjectManager.Instance.getGameObjectsWhere(objectName);
        tracker.findObjects(lst);
        StartCoroutine(Speak(objectName + " was found, please follow the navigator."
            , () => { }));
    }

    private void goBack()
    {
        keywordRecognizer.Stop();
        RealWorldObjectManager.Instance.hideWorldObjects();
        StartCoroutine(goBackAux());
    }

    IEnumerator goBackAux()
    {
        tracker.stopShooting();
        yield return new WaitUntil(() => tracker.isStoppedPhotoMode == true);
        GetComponent<ActiveCallBack>().activeOnFinish.SetActive(true);
        MySocket.log("setting menu up");
        Destroy(gameObject);
    }

    IEnumerator Speak(string msg, Action act)
    {
        textToSpeech.StartSpeaking(msg);
        yield return new WaitUntil(() => textToSpeech.IsSpeaking() == true);
        yield return new WaitUntil(() => textToSpeech.IsSpeaking() == false);
        act();
    }
}
