﻿using HoloToolkit.Unity;
using UnityEngine;

/// <summary>
/// The Interactible class flags a Game Object as being "Interactible".
/// Determines what happens when an Interactible is being gazed at.
/// </summary>
public class Interactible : MonoBehaviour
{
    [Tooltip("Audio clip to play when interacting with this hologram.")]
    public AudioClip TargetFeedbackSound;

    private TextToSpeech textToSpeech;
    [Tooltip("speak this message on tap")]
    public string speakOnTap = null;

    [Tooltip("scal on gaze")]
    public float scalOnGaze = 1.0f;

    private AudioSource audioSource;
    private Material[] defaultMaterials;
    private Vector3 originalScale;

    void Start()
    {
        if (GetComponent<Renderer>() != null)
        {
            defaultMaterials = GetComponent<Renderer>().materials;
        }
        originalScale = gameObject.transform.localScale;
        // Add a BoxCollider if the interactible does not contain one.
        Collider collider = GetComponentInChildren<Collider>();
        if (collider == null)
        {
            gameObject.AddComponent<BoxCollider>();
        }

        EnableAudioHapticFeedback();
    }

    private void Awake()
    {
        textToSpeech = GetComponent<TextToSpeech>();
    }

    private void EnableAudioHapticFeedback()
    {
        // If this hologram has an audio clip, add an AudioSource with this clip.
        if (TargetFeedbackSound != null)
        {
            audioSource = GetComponent<AudioSource>();
            if (audioSource == null)
            {
                audioSource = gameObject.AddComponent<AudioSource>();
            }

            audioSource.clip = TargetFeedbackSound;
            audioSource.playOnAwake = false;
            audioSource.spatialBlend = 1;
            audioSource.dopplerLevel = 0;
        }
    }

    /* TODO: DEVELOPER CODING EXERCISE 2.d */

    void GazeEntered()
    {
        this.gameObject.transform.localScale *= scalOnGaze;
        // Camera.main.gameObject.SendMessage("onGazeEntered", this.gameObject);
        /*for (int i = 0; i < defaultMaterials.Length; i++)
        {
            // 2.d: Uncomment the below line to highlight the material when gaze enters.
            defaultMaterials[i].SetFloat("_Highlight", .25f);
        }*/
    }

    void GazeExited()
    {
        this.gameObject.transform.localScale /= scalOnGaze;
        /*for (int i = 0; i < defaultMaterials.Length; i++)
        {
            // 2.d: Uncomment the below line to remove highlight on material when gaze exits.
            defaultMaterials[i].SetFloat("_Highlight", 0f);
        }*/
    }

    public void resetGaze()
    {
        this.gameObject.transform.localScale = originalScale;
    }

    void OnSelect()
    {
        /*for (int i = 0; i < defaultMaterials.Length; i++)
        {
            defaultMaterials[i].SetFloat("_Highlight", .5f);
        }*/

        // Play the audioSource feedback when we gaze and select a hologram.
        MySocket.log("selecting");
        if (!string.IsNullOrEmpty(speakOnTap) && !textToSpeech.IsSpeaking())
        {
            MySocket.log("speaking gameobject name");
            textToSpeech.StartSpeaking(speakOnTap);
        }

        /*if (audioSource != null && !audioSource.isPlaying)
        {
            audioSource.Play();
        }*/

        /* TODO: DEVELOPER CODING EXERCISE 6.a */
        // 6.a: Handle the OnSelect by sending a PerformTagAlong message.
        
    }

}