﻿#if UNITY_UWP && !UNITY_EDITOR
using System.Threading.Tasks;
#endif
using UnityEngine;
using System.Collections.Generic;
using System.Linq;
using UnityEngine.VR.WSA.WebCam;
using HoloToolkit.Unity.SpatialMapping;
using HoloToolkit.Unity;
using UnityEngine.Windows.Speech;
using System;
using System.Collections;
using Academy.HoloToolkit.Unity;

public class ObjectTracking : MonoBehaviour {

    PhotoCapture photoCaptureObject = null;
    Dictionary<string, FrameData> image2Matrices;
    List<Vector3> positions = new List<Vector3>();
    TriggerTracking trigger;
    GameState state;
    int imageWidth = 0;
    int imageHeight = 0;
    int numberOfPhotos = 0;
    int max_failed_first_pics = 0;

    Queue<KeyValuePair<string, byte[]>> imagesToProcess = new Queue<KeyValuePair<string, byte[]>>();
    bool processedImage = false;

    bool firstPic = true;
    bool canSendRequests = true;
    DynamicTracker tracker;
    private TextToSpeech textToSpeech;
    ObjectDetector detector;

    // Use this for initialization
    void Start() {
        image2Matrices = new Dictionary<string, FrameData>();
        trigger = new TriggerTracking();
        state = GameState.Sleep;
        StartCapturing();    
        MySocket.ignore  = false;
        MySocket.isPositionReady = false;
        MySocket.resultPos = Vector2.zero;
        detector = new ObjectDetector(() => playFoundSound());
    }

    private void Awake()
    {
        textToSpeech = GetComponent<TextToSpeech>();
    }

    enum GameState
    {
        Sleep, Play
    }

    public void StartCapturing()
    {
        Resolution cameraResolution = PhotoCapture.SupportedResolutions.OrderByDescending((res) => res.width * res.height).Last();
        MySocket.log("Activating Camera " + cameraResolution.width + " X " + cameraResolution.height);
        imageWidth = cameraResolution.width;
        imageHeight = cameraResolution.height;
        if (!MySocket.record)
        {
            PhotoCapture.CreateAsync(false, delegate (PhotoCapture captureObject)
            {
                photoCaptureObject = captureObject;
                CameraParameters cameraParameters = new CameraParameters();
                cameraParameters.hologramOpacity = 0.0f;
                cameraParameters.cameraResolutionWidth = cameraResolution.width;
                cameraParameters.cameraResolutionHeight = cameraResolution.height;
                cameraParameters.pixelFormat = CapturePixelFormat.JPEG;

                // Activate the camera
                photoCaptureObject.StartPhotoModeAsync(cameraParameters, delegate (PhotoCapture.PhotoCaptureResult result)
                {
                    // Take a picture
                });
            });
        }
        MySocket.record = false;
    }

    int count_bad_first_images = 0;
    void ExecutePictureProcess()
    {
        if (state != GameState.Play) return;
        if (processedImage) return;
        if (MySocket.record) return;
        processedImage = true;
        photoCaptureObject.TakePhotoAsync(delegate (PhotoCapture.PhotoCaptureResult result, PhotoCaptureFrame photoCaptureFrame)
        {
            List<byte> buffer = new List<byte>();
            photoCaptureFrame.CopyRawImageDataIntoBuffer(buffer);
            Matrix4x4 lastCameraToWorld;
            Matrix4x4 lastProjection;
            if (!photoCaptureFrame.TryGetCameraToWorldMatrix(out lastCameraToWorld) ||
                 !photoCaptureFrame.TryGetProjectionMatrix(out lastProjection))
            {
                processedImage = false;
                return;
            }
            this.image2Matrices.Add(numberOfPhotos.ToString(), new FrameData()
            {
                lastCameraToWorld = lastCameraToWorld,
                lastProjection = lastProjection
            });
            imagesToProcess.Enqueue(new KeyValuePair<string, byte[]>(numberOfPhotos.ToString(), buffer.ToArray()));
            numberOfPhotos++;
            processedImage = false;
        });
    }

    Vector3 get3DFromImage(string imageId, Vector2 pos)
    {
        if (pos == Vector2.zero)
        {
            MySocket.log("OOV");
            return Vector3.zero;
        }
        if (pos.x < 0 || pos.y < 0)
        {
            if (GazeManager.Instance.SpatialMappingHit)
            {
                return GazeManager.Instance.SpatialMappingPosition;
            }
            return Vector3.zero;
        }
        Vector3 realPos = get3DPosition(pos, imageWidth, imageHeight, this.image2Matrices[imageId].lastProjection,
                        this.image2Matrices[imageId].lastCameraToWorld);
        return realPos;
    }

    Vector2 UploadFirstImage(byte[] imageBuffer, Vector2 pixelPos)
    {
        MySocket.sendFrameForTracker(imageBuffer, numberOfPhotos.ToString(), pixelPos);
        return MySocket.readBoundBox();
    }

    List<GameObject> foundObjects = null;
    enum VisualState
    {
        Find,
        NearestObject
    }

    public void findObjects(List<GameObject> lst)
    {
        detector.navigateTo(lst);
    }

    public AudioClip onFindMusic;
    void playFoundSound()
    {
        if (onFindMusic != null)
        {
            AudioSource src = GetComponent<AudioSource>();
            src.clip = onFindMusic;
            src.Play();
        }        
    }

    void Update() {
        detector.Update();
        if (state == GameState.Sleep)
        {
            trigger.Update();
            if (trigger.startTracking)
            {
                BeginTracking();
                InvokeRepeating("ExecutePictureProcess", 0.1f, 0.1f);
                state = GameState.Play;
            }
        }
        else if (state == GameState.Play)
        {
            if (MySocket.isPositionReady)
            {
                MySocket.log("got feedback");
                MySocket.isPositionReady = false;
                var IdImageToBytes = imagesToProcess.Dequeue();
                if (firstPic)
                {
                    MySocket.log("recieved first pic results");
                    if (MySocket.resultPos != Vector2.zero)
                    {
                        MySocket.log("good pixels");
                        count_bad_first_images = 0;
                        firstPic = false;
                    }
                    else
                    {
                        MySocket.log("bad pixels");
                        count_bad_first_images++;
                        if (count_bad_first_images > 10)
                        {
                            MySocket.log("gave up tracking");
                            EndTracking();
                        }
                        firstPic = true;
                    }
                }
                else
                {
                    Vector3 newPos = get3DFromImage(IdImageToBytes.Key, MySocket.resultPos);
                    tracker.addPosition(newPos);
                }
                canSendRequests = true;
                MySocket.log("finished process feedback");
            }
            if (canSendRequests && imagesToProcess.Count > 0)
            {
                canSendRequests = false;
                var IdImageToBytes = imagesToProcess.Peek();

                if (firstPic)
                {
                    Vector3 pos3D = tracker.trackedObject.transform.position;
                    if (GazeManager.Instance.SpatialMappingHit)
                    {
                        pos3D = GazeManager.Instance.SpatialMappingPosition;
                    }
                    Vector2 pixelPos = get2DPos(pos3D,
                        this.image2Matrices[IdImageToBytes.Key].lastProjection,
                        this.image2Matrices[IdImageToBytes.Key].lastCameraToWorld);
                    if (pixelPos.x >= 0 && pixelPos.y >= 0)
                    {
                        MySocket.log("Sending first image");
#if UNITY_UWP && !UNITY_EDITOR
                Task.Run(() => MySocket.UploadFirstImage(IdImageToBytes.Value, IdImageToBytes.Key, pixelPos));
#endif
                    }
                    else
                    {
                        canSendRequests = true;
                        count_bad_first_images++;
                        if (count_bad_first_images > 10)
                        {
                            MySocket.log("gave up tracking");
                            EndTracking();
                        }
                        MySocket.log("failed to send first pic");
                    }
                }
                else
                {
#if UNITY_UWP && !UNITY_EDITOR
                Task.Run(() => MySocket.processTrackImage(IdImageToBytes.Value, IdImageToBytes.Key));
#endif
                }
            }
            tracker.Update();
            if (tracker.state == DynamicTracker.TrackerState.OnTheMove)
            {
                
            }
            if (tracker.state == DynamicTracker.TrackerState.Done)
            {
                if (tracker.result == DynamicTracker.TrackerResult.Remove)
                {
                    string objectName = RealWorldObjectManager.Instance.getWorldObject(tracker.trackedObject);
                    StartCoroutine(RealWorldObjectManager.Instance.removeObject(tracker.trackedObject));
                    detector.removeObject(tracker.trackedObject);
                    StartCoroutine(Speak(objectName + " was removed."
                        , () => { }));

                }
                else if (tracker.result == DynamicTracker.TrackerResult.Change)
                {
                    string objectName = RealWorldObjectManager.Instance.getWorldObject(tracker.trackedObject);
                    StartCoroutine(RealWorldObjectManager.Instance.changeObjectLocation(tracker.trackedObject,
                        tracker.finalPosition));
                    detector.addObject(tracker.trackedObject);
                    StartCoroutine(Speak(objectName + " location changed."
                        , () => { }));
                }
                EndTracking();
            }
        }
	}

    void BeginTracking()
    {
        firstPic = true;
        canSendRequests = true;
        MySocket.ignore = false;
        MySocket.isPositionReady = false;
        tracker = new DynamicTracker(trigger.suspect);
    }

    IEnumerator Speak(string msg, Action act)
    {
        textToSpeech.StartSpeaking(msg);
        yield return new WaitUntil(() => textToSpeech.IsSpeaking() == true);
        yield return new WaitUntil(() => textToSpeech.IsSpeaking() == false);
        act();
    }

    void EndTracking()
    {
        CancelInvoke("ExecutePictureProcess");
        state = GameState.Sleep;
        trigger = new TriggerTracking();
        imagesToProcess.Clear();
        count_bad_first_images = 0;
        MySocket.ignore = true;
    }

    Vector3 get3DPosition(Vector2 PixelPos, int ImageWidth, int ImageHeight, Matrix4x4 proj
       , Matrix4x4 CameraToWorld)
    {
        Vector2 ImagePosZeroToOne = new Vector2(PixelPos.x / ImageWidth, 1.0f - (PixelPos.y / ImageHeight));
        Vector2 ImagePosProjected = ((ImagePosZeroToOne * 2.0f) - new Vector2(1, 1)); // -1 to 1 space
        Vector3 CameraSpacePos = UnProjectVector(proj, new Vector3(ImagePosProjected.x, ImagePosProjected.y, 1));
        Vector3 cameraPosition = CameraToWorld.MultiplyPoint(Vector3.zero);
        Vector3 rayPosition = CameraToWorld.MultiplyPoint(CameraSpacePos);
        return rayPosition;
    }

    Vector2 get2DPos(Vector3 WorldSpacePos, Matrix4x4 CameraProjection, Matrix4x4 CameraToWorld)
    {
        // Usual 3d math:
        Matrix4x4 WorldToCamera = CameraToWorld.inverse;
        Vector4 CameraSpacePos = WorldToCamera * new Vector4(WorldSpacePos.x, WorldSpacePos.y, WorldSpacePos.z, 1); // use 1 as the W component                                                                                                            // Projection math:
        Vector4 ImagePosUnnormalized = CameraProjection * new Vector4(CameraSpacePos.x, CameraSpacePos.y, CameraSpacePos.z, 1); // use 1 as the W component
        Vector2 ImagePosProjected = new Vector2(ImagePosUnnormalized.x, ImagePosUnnormalized.y) / ImagePosUnnormalized.w; // normalize by W, gives -1 to 1 space
        Vector2 ImagePosZeroToOne = (ImagePosProjected * 0.5f) + new Vector2(0.5f, 0.5f); // good for GPU textures
        Vector2 PixelPos = new Vector2(ImagePosZeroToOne.x * imageWidth, (1 - ImagePosZeroToOne.y) * imageHeight); // good for CPU textures*/
        return PixelPos;
    }

    public static Vector3 UnProjectVector(Matrix4x4 proj, Vector3 to)
    {
        Vector3 from = new Vector3(0, 0, 0);
        var axsX = proj.GetRow(0);
        var axsY = proj.GetRow(1);
        var axsZ = proj.GetRow(2);
        from.z = to.z / axsZ.z;
        from.y = (to.y - (from.z * axsY.z)) / axsY.y;
        from.x = (to.x - (from.z * axsX.z)) / axsX.x;
        return from;
    }

    public bool isStoppedPhotoMode = false;

    public void stopShooting()
    {
        if (!MySocket.record)
        {
            photoCaptureObject.StopPhotoModeAsync(OnStoppedPhotoMode);
        }
    }

    void OnStoppedPhotoMode(PhotoCapture.PhotoCaptureResult result)
    {
        MySocket.log("Stopped shooting tracker");
        photoCaptureObject.Dispose();
        photoCaptureObject = null;
        isStoppedPhotoMode = true;
    }

    class FrameData
    {
        public Matrix4x4 lastCameraToWorld;
        public Matrix4x4 lastProjection;
    }
}
class ObjectDetector
{
    float thresDistanceFromSuspect = 0.1f;
    int thresFrames = 30;
    int surviveFrames = 180;
    int count = 0;
    private GameObject suspect;
    List<GameObject> navigateObjects = new List<GameObject>();
    List<GameObjectCounter> gameObjToCounter = new List<GameObjectCounter>();
    vState state = vState.NearestObject;
    Action actOnFound;

    enum vState
    {
        Find,
        NearestObject,
    }

    public ObjectDetector(Action actOnFound)
    {
        suspect = null;
        this.actOnFound = actOnFound;
    }

    bool seenObject()
    {
        if (GazeManager.Instance.Hit)
        {
            if (suspect == null)
            {
                suspect = GazeManager.Instance.HitObject;
                return true;
            }
            return suspect == GazeManager.Instance.HitObject;
        }
        if (GazeManager.Instance.SpatialMappingHit)
        {
            if (suspect == null)
            {
                suspect = RealWorldObjectManager.Instance.getClosesObject(GazeManager.Instance.SpatialMappingPosition
                                            , thresDistanceFromSuspect);
                if (suspect == null) return false;
            }
            return Vector3.Distance(suspect.transform.position, GazeManager.Instance.SpatialMappingPosition) <= thresDistanceFromSuspect;
        }
        return false;
    }

    class GameObjectCounter
    {
        public GameObject gameObj { get; set; }
        public int count { get; set; }
    }

    public void navigateTo(List<GameObject> foundObjs)
    {
        state = vState.Find;
        navigateObjects.Clear();
        gameObjToCounter.Clear();
        count = 0;
        RealWorldObjectManager.Instance.hideWorldObjects();
        foreach (var gameobj in foundObjs)
        {
            navigateObjects.Add(gameobj);
            RealWorldObjectManager.Instance.showWorldObject(gameobj, true);
        }
    }

    public void Update()
    {
        if (seenObject())
        {
            if (state == vState.Find)
            {
                if (navigateObjects.Contains(suspect))
                {
                    actOnFound();
                    state = vState.NearestObject;
                    RealWorldObjectManager.Instance.hideWorldObjects();
                    addObject(suspect);
                }
            }
            else
            {
                count++;
                if (count >= thresFrames)
                {
                    addObject(suspect);
                }
            }
        }
        else
        {
            suspect = null;
            count = 0;
        }
        UpdateCounters();
    }

    public void addObject(GameObject gameObj)
    {
        foreach (var gamecounter in gameObjToCounter)
        {
            if (gameObj == gamecounter.gameObj)
            {
                gamecounter.count = surviveFrames;
                return;
            }
        }
        gameObjToCounter.Add(new GameObjectCounter()
        {
            gameObj = gameObj,
            count = surviveFrames
        });
        RealWorldObjectManager.Instance.showWorldObject(gameObj, false);
    }

    public void removeObject(GameObject gameObj)
    {
        gameObjToCounter.RemoveAll(s => s.gameObj == gameObj);
        navigateObjects.RemoveAll(s => s == gameObj);
    }

    void UpdateCounters()
    {
        List<GameObject> toRemove = new List<GameObject>();
        foreach (var gamecounter in gameObjToCounter)
        {
            gamecounter.count--;
            if (gamecounter.count <= 0)
            {
                toRemove.Add(gamecounter.gameObj);
            }
        }
        toRemove.ForEach(s => RealWorldObjectManager.Instance.hideWorldObject(s));
        gameObjToCounter.RemoveAll(s => toRemove.Contains(s.gameObj));
    }
}
class TriggerTracking
{
    float thresDistanceFromSuspect = 0.1f;
    float thresDistanceFromCameraToSuspect = 0.5f;
    int thresFrames = 60; //was 100
    int count = 0;
    public GameObject suspect { get; set; }
    public bool startTracking { get; set; }

    public TriggerTracking()
    {
        suspect = null;
        startTracking = false;
    }

    bool seenObject()
    {
        if (GazeManager.Instance.Hit)
        {
            if (suspect == null)
            {
                suspect = GazeManager.Instance.HitObject;
                return true;
            }
            return suspect == GazeManager.Instance.HitObject;
        }
        if (GazeManager.Instance.SpatialMappingHit)
        {
            if (suspect == null)
            {
                suspect = RealWorldObjectManager.Instance.getClosesObject(GazeManager.Instance.SpatialMappingPosition
                                            , thresDistanceFromSuspect);
                if (suspect == null) return false;
            }
            return Vector3.Distance(suspect.transform.position, GazeManager.Instance.SpatialMappingPosition) <= thresDistanceFromSuspect;
        }
        return false;
    }

    public void Update()
    {
        if (startTracking) return;
        if (seenObject())
        {

            Vector2 pos1 = new Vector2(Camera.main.transform.position.x, Camera.main.transform.position.z);
            Vector2 pos2 = new Vector2(suspect.gameObject.transform.position.x, suspect.gameObject.transform.position.z);
            if (Vector2.Distance(pos1, pos2) <= thresDistanceFromCameraToSuspect)
            {
                count++;
                if (count >= thresFrames)
                {
                    startTracking = true;
                }
            }
            else
            {
                count = 0;
            }
        }
        else
        {
            suspect = null;
            count = 0;
        }
    }
}
class DynamicTracker
{
    float thresDistance = 0.1f;
    int max_positions = 4;

    int count_oov_frames = 0;
    int max_oov_frames = 60;

    public GameObject trackedObject { get; set; }
    public Vector3 finalPosition { get; set; }
    public List<Vector3> positions { get; set; }
    public Vector3 initialPosition { get; set; }

    public TrackerState state { get; set; }
    public TrackerResult result { get; set; }

    public enum TrackerState
    {
        Before,
        OnTheMove,
        Done
    }

    public enum TrackerResult
    {
        Change,
        Remove,
        Idle
    }

    public DynamicTracker(GameObject suspect)
    {
        this.trackedObject = suspect;
        finalPosition = Vector3.zero;
        positions = new List<Vector3>();
        initialPosition = suspect.transform.position;
        state = TrackerState.Before;
    }

    void handleMovingObject()
    {
        Vector3 firstPosition = positions.First();
        bool isSameLocation = true;
        foreach (var pos in positions)
        {
            if (pos == Vector3.zero ||
                Vector3.Distance(pos, firstPosition) > thresDistance)
            {
                isSameLocation = false;
                break;
            }
        }
        if (isSameLocation)
        {
            Vector3 avgPos = positions.Average();
            RaycastHit hitInfo = new RaycastHit();
            if (Physics.Raycast(
                Camera.main.transform.position, avgPos - Camera.main.transform.position, out hitInfo,
                30.0f, SpatialMappingManager.Instance.LayerMask)
                && get2DDistance(Camera.main.transform.position, hitInfo.point) < 0.5f) 
            {
                MySocket.log("Finished tracking, on change");
                result = TrackerResult.Change;
                state = TrackerState.Done;
                finalPosition = hitInfo.point;
            }
        }
    }

    float get2DDistance(Vector3 a, Vector3 b)
    {
        Vector2 pos1 = new Vector2(a.x, a.z);
        Vector2 pos2 = new Vector2(b.x, b.z);
        return Vector2.Distance(pos1, pos2);
    }

    bool viewdOnce = false;
    Vector3 spatialMappingHitPos;

    public void Update()
    {
        if (state == TrackerState.Done) return;
        if (positions.Count > max_positions)
        {
            positions.RemoveAt(0);
        }
        if (state == TrackerState.Before && positions.Count > 0)
        {
            if (positions.Last() != Vector3.zero)
            {
                RaycastHit hitInfo = new RaycastHit();
                if (Physics.Raycast(
                    Camera.main.transform.position, positions.Last() - Camera.main.transform.position, out hitInfo,
                    30.0f, SpatialMappingManager.Instance.LayerMask)
                    && Vector3.Distance(initialPosition, hitInfo.point) > 0.3f)
                {
                    MySocket.log("switching to On the move");
                    state = TrackerState.OnTheMove;
                }
            }
            /*
            if (state == TrackerState.Before &&
                positions.Count >= 2 && positions.Last() == Vector3.zero &&
                positions[positions.Count-2] != Vector3.zero &&
                Vector3.Distance(spatialMappingHitPos, GazeManager.Instance.SpatialMappingPosition) < 0.3f)
            {
                MySocket.log("switching to On the move new cond");
                state = TrackerState.OnTheMove;
            }*/
        }
        else if (state == TrackerState.OnTheMove)
        {
            handleMovingObject();   
        }
        if (count_oov_frames > max_oov_frames)
        {
            MySocket.log("oov for too long");
            if (state == TrackerState.Before)
            {
                MySocket.log("Idle");
                result = TrackerResult.Idle;
            }
            else if (state == TrackerState.OnTheMove)
            {
                MySocket.log("Remove");
                result = TrackerResult.Remove;
            }
            state = TrackerState.Done;
            finalPosition = Vector3.zero;
        } 
    }

    Vector3 getReal3DPos(Vector3 direction)
    {
        if (direction == Vector3.zero) return direction;
        RaycastHit hitInfo = new RaycastHit();
        if (Physics.Raycast(
            Camera.main.transform.position, direction - Camera.main.transform.position, out hitInfo,
            30.0f, SpatialMappingManager.Instance.LayerMask))
        {
            return hitInfo.point;
        }
        return Vector3.zero;
    }

    public void addPosition(Vector3 newPos)
    {
        if (newPos == Vector3.zero)
        {
            count_oov_frames++;
        }
        else
        {
            spatialMappingHitPos = GazeManager.Instance.SpatialMappingPosition;
            if (this.positions.Count > 0 && this.positions.Last() != Vector3.zero)
            {
                count_oov_frames = 0;
            }
        }
        this.positions.Add(getReal3DPos(newPos));
    }
}