﻿#if UNITY_UWP && !UNITY_EDITOR
using System.Threading.Tasks;
#endif
using UnityEngine;
using System.Collections.Generic;
using System.Linq;
using UnityEngine.VR.WSA.WebCam;
using HoloToolkit.Unity.SpatialMapping;
using HoloToolkit.Unity;
using UnityEngine.Windows.Speech;
using System;
using System.Collections;
using Academy.HoloToolkit.Unity;

public class PhotoCaptureMgr : MonoBehaviour
{

    PhotoCapture photoCaptureObject = null;
    int numberOfPhotos = 0;
    bool isProcessingImage = false;
    Dictionary<string, FrameData> image2Matrices;
    KeywordRecognizer keywordRecognizer = null;
    ScannerInteraction scannerMain;
    List<RayPoints> rays;

    int cameraResolutionWidth;
    int cameraResolutionHeight;

    public AudioClip takingPictureEffect;
    bool stopPhoto = false;
    Mode mode = Mode.Voice;

    enum Mode
    {
        Voice,
        Gaze
    }

    // Use this for initialization
    void Start()
    {
        image2Matrices = new Dictionary<string, FrameData>();
        rays = new List<RayPoints>();
    }

    

    public void StartCapturing()
    {
        Resolution cameraResolution = PhotoCapture.SupportedResolutions.OrderByDescending((res) => res.width * res.height).First();
        log("Activating Camera " + cameraResolution.width + " X " + cameraResolution.height);
        cameraResolutionWidth = cameraResolution.width;
        cameraResolutionHeight = cameraResolution.height;
        // Create a PhotoCapture object

        keywordRecognizer = new KeywordRecognizer(new string[] { "Stop", "Take Photo" , "Change", "Clean" });
        keywordRecognizer.OnPhraseRecognized += stopShooting;
        keywordRecognizer.Start();

        if (!MySocket.record)
        {
            PhotoCapture.CreateAsync(false, delegate (PhotoCapture captureObject) {
                photoCaptureObject = captureObject;
                CameraParameters cameraParameters = new CameraParameters();
                cameraParameters.hologramOpacity = 0.0f;
                cameraParameters.cameraResolutionWidth = cameraResolution.width;
                cameraParameters.cameraResolutionHeight = cameraResolution.height;
                cameraParameters.pixelFormat = CapturePixelFormat.JPEG;

                // Activate the camera
                photoCaptureObject.StartPhotoModeAsync(cameraParameters, delegate (PhotoCapture.PhotoCaptureResult result) {
                    // Take a picture
                    // InvokeRepeating("ExecutePictureProcess", 3.0f, 0.25f);
                });
            });
        }
    }

    private void Awake()
    {
        scannerMain = GetComponent<ScannerInteraction>();
    }

    void Update()
    {
        if (!stopPhoto && mode == Mode.Gaze && isSpatialMappingOn())
        {
            TakePhotoOnStaring();
        }
        if (MySocket.isBoxesReady && isSpatialMappingOn())
        {
            MySocket.isBoxesReady = false;
            MySocket.log("going in");
            foreach (var image_id in MySocket.image2Objects.Keys)
            {
                foreach (var detectedObject in MySocket.image2Objects[image_id])
                {
                    addRay(detectedObject.objectPos, cameraResolutionWidth, cameraResolutionHeight,
                        this.image2Matrices[image_id].lastProjection,
                        this.image2Matrices[image_id].lastCameraToWorld, detectedObject);
                }
            }
            MySocket.log("finished loop1");
            foreach (var ray in rays)
            {
                RaycastHit hitInfo = new RaycastHit();
                if (Physics.Raycast(
                    ray.origin, ray.direction, out hitInfo,
                    30.0f, SpatialMappingManager.Instance.LayerMask))
                {
                    ray.detectedObject.object3DPos = hitInfo.point;
                }
            }
            MySocket.log("finished loop2");
            RealWorldObjectManager.Instance.onRealWorldObjectsFound(rays);
            SendMessage("onObjectsMapped");
            MySocket.clearReadBoxes();
            MySocket.log("finished processing rays");
        }
    }

    int count = 0;
    float thresDistance = 0.5f;
    int thresFrames = 100;
    bool firstPic = false;
    Vector3 gazePosition;
    Vector3 cameraPosition;

    void TakePhotoOnStaring()
    {
        if (GazeManager.Instance.SpatialMappingHit)
        {
            if (count == 0)
            {
                cameraPosition = Camera.main.transform.position;
                gazePosition = GazeManager.Instance.SpatialMappingPosition;
            }
            if (Vector3.Distance(cameraPosition, Camera.main.transform.position) <= thresDistance
                    && Vector3.Distance(gazePosition, GazeManager.Instance.SpatialMappingPosition) <= thresDistance)
            {
                count++;
            }
            else
            {
                count = 0;
            }
            if (count >= thresFrames)
            {
                StartCoroutine(playPhotoSound());
                ExecutePictureProcess();
                count = 0;
            }
        }
        else
        {
            count = 0;
        }
    }
    void stopShooting(PhraseRecognizedEventArgs args)
    {
        if (stopPhoto) return;
        if (args.text == "Clean")
        {
            StartCoroutine(RealWorldObjectManager.Instance.clean(() =>
            {
                GetComponent<ActiveCallBack>().activeOnFinish.SetActive(true);
                photoCaptureObject.StopPhotoModeAsync(OnStoppedPhotoMode);
                stopPhoto = true;
                MySocket.isBoxesReady = false;
                Destroy(gameObject);
            }));
        }
        if (args.text == "Stop")
        {
            MySocket.log("Stopping");
            MySocket.sendFrame(new byte[] { 0, 0, 0 }, numberOfPhotos.ToString(), true);
            scannerMain.stopSound();
            if (!MySocket.record) {
                photoCaptureObject.StopPhotoModeAsync(OnStoppedPhotoMode);
            }
            SendMessage("onScanFinished");
            stopPhoto = true;
#if UNITY_UWP && !UNITY_EDITOR
            Task.Run(() => MySocket.readBoxes());
#endif
        }
        else if (args.text == "Change")
        {
            string modeName = "";
            Mode newMd = Mode.Voice;
            if (mode == Mode.Voice) {
                newMd = Mode.Gaze;
                modeName = "gaze.";
            }
            else if (mode == Mode.Gaze) {
                newMd = Mode.Voice;
                modeName = "voice.";
            }
            count = 0;
            MySocket.log("Changing!");
            speakAndStopSound("Changing to " + modeName, () =>
            {
                mode = newMd;
            });
        }
        else if (args.text == "Take Photo" && mode == Mode.Voice)
        {
            StartCoroutine(playPhotoSound());
            ExecutePictureProcess();
        }
        
    }

    IEnumerator playPhotoSound()
    {
        scannerMain.stopSound();
        AudioSource src = GetComponent<AudioSource>();
        src.clip = takingPictureEffect;
        src.Play();
        yield return new WaitForSeconds(src.clip.length);
        scannerMain.startSound();
    }

    void speakAndStopSound(string msg, Action act)
    {
        scannerMain.stopSound();
        StartCoroutine(scannerMain.Speak(msg, () =>
        {
            scannerMain.startSound();
            act();
        }));
    }

    bool isSpatialMappingOn()
    {
        var headPosition = Camera.main.transform.position;
        var gazeDirection = Camera.main.transform.forward;
        RaycastHit hitInfo;
        return Physics.Raycast(headPosition, gazeDirection, out hitInfo,
            30.0f, SpatialMappingManager.Instance.LayerMask);
    }

    List<GameObject> images = new List<GameObject>();

    void addImageToRealWorld(PhotoCaptureFrame photoCaptureFrame, byte[] imageRaw
        , Matrix4x4 cameraToWorldMatrix)
    {
        MySocket.log("Started adding image");
        Texture2D m_Texture = new Texture2D(cameraResolutionWidth, cameraResolutionHeight);
        if (!m_Texture.LoadImage(imageRaw.ToArray(), false))
        {
            MySocket.log("failed to add image");
        }
        GameObject m_Quad = GameObject.CreatePrimitive(PrimitiveType.Quad);
        MySocket.log("p0");
        Renderer m_QuadRenderer = m_Quad.GetComponent<Renderer>() as Renderer;
        MySocket.log("p1");
        m_QuadRenderer.material = new Material(Shader.Find("Custom/Unlit/UnlitTexture"));
        MySocket.log("p2");
        m_QuadRenderer.material.SetTexture("_MainTex", m_Texture);
        MySocket.log("p3");
        Vector3 position = cameraToWorldMatrix.MultiplyPoint(Vector3.zero);
        MySocket.log("p4");
        Quaternion rotation = Quaternion.LookRotation(-cameraToWorldMatrix.GetColumn(2), cameraToWorldMatrix.GetColumn(1));
        m_Quad.transform.position = position;
        m_Quad.transform.rotation = rotation;
        images.Add(m_Quad);
        MySocket.log("stopped adding image");
    }

    void ExecutePictureProcess()
    {
        if (isProcessingImage) return;
        if (MySocket.record) return;
        isProcessingImage = true;
        //Take a picture
        photoCaptureObject.TakePhotoAsync(delegate (PhotoCapture.PhotoCaptureResult result, PhotoCaptureFrame photoCaptureFrame) {
            List<byte> buffer = new List<byte>();
            photoCaptureFrame.CopyRawImageDataIntoBuffer(buffer);
            Matrix4x4 lastCameraToWorld;
            Matrix4x4 lastProjection;
            if (!photoCaptureFrame.TryGetCameraToWorldMatrix(out lastCameraToWorld) ||
                 !photoCaptureFrame.TryGetProjectionMatrix(out lastProjection))
            {
                isProcessingImage = false;
                return;
            }
            // addImageToRealWorld(photoCaptureFrame, buffer.ToArray(), lastCameraToWorld);
            this.image2Matrices.Add(numberOfPhotos.ToString(), new FrameData()
            {
                lastCameraToWorld = lastCameraToWorld,
                lastProjection = lastProjection
            });
            UploadImage(buffer.ToArray());
        });   
    }

    public void addRay(Vector2 PixelPos, int ImageWidth, int ImageHeight, Matrix4x4 proj
        , Matrix4x4 CameraToWorld, DetectedObjectData obj)
    {
        Vector2 ImagePosZeroToOne = new Vector2(PixelPos.x / ImageWidth, 1.0f - (PixelPos.y / ImageHeight));
        Vector2 ImagePosProjected = ((ImagePosZeroToOne * 2.0f) - new Vector2(1, 1)); // -1 to 1 space
        Vector3 CameraSpacePos = UnProjectVector(proj, new Vector3(ImagePosProjected.x, ImagePosProjected.y, 1));
        Vector3 cameraPosition = CameraToWorld.MultiplyPoint(Vector3.zero);
        Vector3 rayPosition = CameraToWorld.MultiplyPoint(CameraSpacePos);
        rays.Add(new RayPoints()
        {
            origin = cameraPosition,
            direction = rayPosition - cameraPosition,
            detectedObject = obj
        });
    }
    
    public static Vector3 UnProjectVector(Matrix4x4 proj, Vector3 to)
    {
        Vector3 from = new Vector3(0, 0, 0);
        var axsX = proj.GetRow(0);
        var axsY = proj.GetRow(1);
        var axsZ = proj.GetRow(2);
        from.z = to.z / axsZ.z;
        from.y = (to.y - (from.z * axsY.z)) / axsY.y;
        from.x = (to.x - (from.z * axsX.z)) / axsX.x;
        return from;
    }
    
    void UploadImage(byte[] imageBuffer)
    {
        log("Uploading image");
        MySocket.sendFrame(imageBuffer, numberOfPhotos.ToString(), false);
        numberOfPhotos++;
        isProcessingImage = false;
    }

    void OnStoppedPhotoMode(PhotoCapture.PhotoCaptureResult result)
    {
        log("Stopped Shooting");
        photoCaptureObject.Dispose();
        photoCaptureObject = null;
    }

    void log(string msg)
    {
        MySocket.log(msg);
    }

    private void OnDestroy()
    {
        foreach (var markerImage in this.images)
        {
            Destroy(markerImage);
        }
    }

    class FrameData
    {
        public Matrix4x4 lastCameraToWorld;
        public Matrix4x4 lastProjection;
    }
}
public class RayPoints
{
    public Vector3 origin;
    public Vector3 direction;
    public DetectedObjectData detectedObject;
}
public class DetectedObjectData
{
    public Vector2 objectPos;
    public Vector3 object3DPos;
    public string label;
    public int score;
}
