import numpy as np
from PIL import Image
import ast
import io
import subprocess
import matplotlib.pyplot as plt

class ObjectDetection:

    YOLO_RUN_CMD = 'darknet_no_gpu.exe detector test cfg/combine9k.data yolo9000.cfg yolo9000.weights {image_file}'

    def run(self, frames):
        for frame in frames:
            frame.setBoxes(self.run_single_image_yolo(frame.img_file))

    def run_single_image_yolo(self, image_file, save_boxes = []):
        current_cmd = ObjectDetection.YOLO_RUN_CMD.format(image_file=image_file)
        proc = subprocess.Popen(current_cmd.split(' '), stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
        startRecord = False
        boxes = []
        image = Image.open(image_file)
        image_np = self.load_image_into_numpy_array(image)
        (im_width, im_height) = image.size
        def fix_size(size, i):
            size = max(size, 0)
            if i == 0:
                size = min(size, im_width)
            else:
                size = min(size, im_height)
            return size
        for line in io.TextIOWrapper(proc.stdout, encoding="utf-8"):  # or another encoding
            line = line.strip()
            print(line)
            if line == '': continue
            if startRecord:
                arr = line.split(')')
                box = ast.literal_eval(arr[0] + ')')
                object_socre_pair = arr[1].split(':')
                object = object_socre_pair[0]
                if int(object_socre_pair[1].strip().replace('%', '')) < 50:
                    continue
                left, right, top, bottom = box
                left -= 80
                right += 30
                top -= 20
                bottom += 20
                left, right, top, bottom = fix_size(left, 0), fix_size(right, 0), fix_size(top, 1), fix_size(bottom, 1)
                new_image_np = image_np[top:bottom, left:right]
                save_boxes.append(new_image_np)
                boxes.append((str(left + (right-left)/2), str(top + (bottom - top)/2),
                              object,
                              object_socre_pair[1].strip().replace('%', '')))
            if 'Predicted' in line:
                startRecord = True
        return boxes

    def load_image_into_numpy_array(self, image):
        (im_width, im_height) = image.size
        return np.array(image.getdata()).reshape(
          (im_height, im_width, 3)).astype(np.uint8)


    def show_image(self, image_file):
        boxes = []
        names = model.run_single_image_yolo(image_file, boxes)
        for j,b in enumerate(names):
            plt.title(b[-2])
            plt.imshow(boxes[j])
            plt.show()

if __name__ == '__main__':
    model = ObjectDetection()
    model.show_image('tmp\\image0.jpeg')
