import cv2
from CMT import CMT
import numpy as np


class CMTTracker:

	def __init__(self, center_point, im0):
		x, y, w, h = self.get_bbox(center_point, im0)
		tl = (x, y)
		br = (x + w, y + h)
		self.CMT = CMT()
		self.CMT.estimate_scale = True
		self.CMT.estimate_rotation = True
		im_gray0 = cv2.cvtColor(im0, cv2.COLOR_BGR2GRAY)
		print('using', tl, br, 'as init bb')
		print('center at ', ((tl[0] + br[0])/2, (tl[1] + br[1])/2))
		self.CMT.initialise(im_gray0, tl, br)

	def draw_bbox(self, original_image, max_rect):
		tl = max_rect[0], max_rect[1]
		tr = max_rect[0] + max_rect[2], max_rect[1]
		bl = max_rect[0], max_rect[1] + max_rect[3]
		br = max_rect[0] + max_rect[2], max_rect[1] + max_rect[3]
		cv2.line(original_image, tl, tr, (255, 0, 0), 4)
		cv2.line(original_image, tr, br, (255, 0, 0), 4)
		cv2.line(original_image, br, bl, (255, 0, 0), 4)
		cv2.line(original_image, bl, tl, (255, 0, 0), 4)
		cv2.imshow("im", original_image)
		cv2.waitKey(0)

	def get_next_box(self, im, toDraw = False):
		im_gray = cv2.cvtColor(im, cv2.COLOR_BGR2GRAY)
		self.CMT.process_frame(im_gray)
		if self.CMT.has_result and toDraw is True:
			cv2.line(im, self.CMT.tl, self.CMT.tr, (255, 0, 0), 4)
			cv2.line(im, self.CMT.tr, self.CMT.br, (255, 0, 0), 4)
			cv2.line(im, self.CMT.br, self.CMT.bl, (255, 0, 0), 4)
			cv2.line(im, self.CMT.bl, self.CMT.tl, (255, 0, 0), 4)
			cv2.imshow("img", im)
			cv2.waitKey(1)
		if self.CMT.has_result:
			return ((self.CMT.tl[0] + self.CMT.br[0])/2, (self.CMT.tl[1] + self.CMT.br[1])/2)
		if toDraw:
			cv2.imshow("img", im)
			cv2.waitKey(260)
		return 0, 0

	def get_bbox(self, point, image, toDraw = False):
		image = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
		high_thresh, thresh_im = cv2.threshold(image, 0, 255, cv2.THRESH_BINARY + cv2.THRESH_OTSU)
		lowThresh = 0.5 * high_thresh
		edged = cv2.Canny(image, lowThresh, high_thresh)
		_, cnts, _ = cv2.findContours(edged.copy(), cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
		max_area = 1000000
		distance_from_point = 10000
		max_rect = ()
		for c in cnts:
			x, y, w, h = cv2.boundingRect(c)
			if w >= int(0.15 * image.shape[0]) and h >= int(0.15 * image.shape[1]):
				if x < point[0] < x + w and y < point[1] < y + h:
					distance_from_point = 0
					if w * h < max_area:
						max_area = w * h
						max_rect = (x, y, w, h)
				else:
					distance = np.linalg.norm(np.array([x, y]) - np.array([point[0], point[1]]))
					if distance < distance_from_point and distance_from_point != 0 and distance < 0.28 * image.shape[0]:
						distance_from_point = distance
						max_rect = (x, y, w, h)
		if max_rect != ():
			if toDraw:
				self.draw_bbox(image, max_rect)
			return max_rect
		def_w = int(0.40 * image.shape[0])
		def_h = int(0.30 * image.shape[1])
		def_w = min(def_w, def_h)
		def_h = def_w
		print("Taking default")
		return max(point[0] - def_w / 2, 1), max(point[1] - def_h / 2, 1), def_w, def_h

if __name__ == '__main__':
	tracker = CMTTracker((415.0, 194.0), cv2.imread("tmp_tracker\\image3.jpeg"))
	start = 0
	end = 25
	for i in range(start, end):
		try:
		   tracker.get_next_box(cv2.imread("tmp_tracker\\image" + str(i) + ".jpeg"), True)
		except Exception as e:
			print(str(e))
			exit()
